@@ -2,26 +2,20 @@
  * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * http://www.freedesktop.org/standards/systemtray.html
  *
@@ -150,19 +144,19 @@ netk_tray_icon_realize (GtkWidget * widget)
     GdkScreen *screen;
 #endif
 
-	/* */
-	GTK_WIDGET_CLASS (parent_class)->realize (widget);
+    /* */
+    GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
 #if defined (GTK_CHECK_VERSION) && GTK_CHECK_VERSION(2,2,0)
-	screen = gtk_widget_get_screen(widget);
-	xscreen = GDK_SCREEN_XSCREEN(screen);
+    screen = gtk_widget_get_screen(widget);
+    xscreen = GDK_SCREEN_XSCREEN(screen);
 #else
-	xscreen = DefaultScreenOfDisplay(GDK_DISPLAY());
+    xscreen = DefaultScreenOfDisplay(GDK_DISPLAY());
 #endif
 
-	icon = NETK_TRAY_ICON(widget);
+    icon = NETK_TRAY_ICON(widget);
 
-	/* look for a manager out there */
+    /* look for a manager out there */
     (void) g_snprintf (selection, sizeof (selection), "_NET_SYSTEM_TRAY_S%d",
                        XScreenNumberOfScreen (xscreen));
 
@@ -267,7 +261,7 @@ netk_tray_icon_update (NetkTrayIcon * icon)
     }
 
     XUngrabServer (xdisplay);
-    XSync (xdisplay, False);
+    XFlush (xdisplay);
 
     if (icon->tray != None)
     {
@@ -320,8 +314,10 @@ netk_tray_icon_opcode (NetkTrayIcon * icon, Window window, glong data1,
     xev.data.l[3] = data3;
     xev.data.l[4] = data4;
 
+    gdk_error_trap_push ();
     XSendEvent (xdisplay, icon->tray, False, NoEventMask, (XEvent *) & xev);
     XSync (xdisplay, FALSE);
+    gdk_error_trap_pop ();
 }
 
 void
@@ -399,6 +395,7 @@ netk_tray_icon_message_new (NetkTrayIcon * icon, glong timeout,
     xev.format = 8;
     xev.message_type = icon->atoms.data;
 
+    gdk_error_trap_push ();
     while (length > 0)
     {
         n = MIN (length, 20);
@@ -412,7 +409,7 @@ netk_tray_icon_message_new (NetkTrayIcon * icon, glong timeout,
 
         XSync (xdisplay, False);
     }
-
+    gdk_error_trap_pop ();
     return (id);
 }
 
