@@ -1428,8 +1428,7 @@ update_state (NetkWindow * window)
             /* Skip taskbar if the window is transient
              * for some main application window
              */
-            if (window->priv->transient_for != None &&
-                !window->priv->transient_for_root)
+            if (window->priv->transient_for)
                 window->priv->skip_taskbar = TRUE;
             break;
 
@@ -1739,12 +1738,8 @@ force_update_now (NetkWindow * window)
      */
 
     old_name = window->priv->name;
-    old_icon_name = window->priv->icon_name;
     window->priv->name = NULL;
-    window->priv->icon_name = NULL;
-
     update_name (window);
-    update_icon_name (window);
 
     if (window->priv->name == NULL)
         window->priv->name = old_name;
@@ -1755,8 +1750,14 @@ force_update_now (NetkWindow * window)
         g_free (old_name);
     }
 
+    old_icon_name = window->priv->icon_name;
+    window->priv->icon_name = NULL;
+    update_icon_name (window);
+    
     if (window->priv->icon_name == NULL)
+    {
         window->priv->icon_name = old_icon_name;
+    }
     else
     {
         if (old_icon_name == NULL ||
