@@ -21,25 +21,34 @@
 #include <config.h>
 #endif
 
+#include <math.h>
+
 #include <gdk/gdk.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkhbox.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkaspectframe.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 
 #include "xfce_menubutton.h"
 
+#define DEBUGGING 0
+
+/* XXX: this is used by gtkbutton. hope they won't change it soon ... */
+#define CHILD_SPACING 1
+
 /* Forward declarations */
 
 static void xfce_menubutton_class_init (XfceMenubuttonClass * class);
 static void xfce_menubutton_init (XfceMenubutton * menubutton);
 static void xfce_menubutton_finalize (GObject * object);
+
 static void xfce_menubutton_size_allocate (GtkWidget * widget,
-				           GtkAllocation * allocation);
-static void xfce_menubutton_scale (XfceMenubutton * menubutton, int width,
-				   int height);
+					   GtkAllocation * allocation);
+
+static gboolean xfce_menubutton_scale (XfceMenubutton * menubutton);
+
+static void hbox_style_set (GtkWidget * widget, GtkStyle * old);
+static void image_style_set (GtkWidget * widget, GtkStyle * old);
 
 /* Local data */
 
@@ -86,7 +95,7 @@ xfce_menubutton_class_init (XfceMenubuttonClass * class)
     parent_class = gtk_type_class (gtk_button_get_type ());
 
     widget_class->size_allocate = xfce_menubutton_size_allocate;
-    
+
     gobject_class->finalize = xfce_menubutton_finalize;
 }
 
@@ -103,7 +112,6 @@ xfce_menubutton_init (XfceMenubutton * menubutton)
 			TRUE, 0);
 
     menubutton->pb = NULL;
-    menubutton->scaled = FALSE;
     menubutton->icon_size = -1;
 
     menubutton->label = gtk_label_new (NULL);
@@ -111,6 +119,12 @@ xfce_menubutton_init (XfceMenubutton * menubutton)
     gtk_widget_show (menubutton->label);
     gtk_box_pack_start (GTK_BOX (menubutton->hbox), menubutton->label, TRUE,
 			TRUE, 0);
+
+    g_signal_connect (menubutton->hbox, "style-set",
+		      G_CALLBACK (hbox_style_set), NULL);
+
+    g_signal_connect (menubutton->image, "style-set",
+		      G_CALLBACK (image_style_set), NULL);
 }
 
 GtkWidget *
@@ -170,14 +184,12 @@ xfce_menubutton_set_pixbuf (XfceMenubutton * menubutton, GdkPixbuf * pixbuf)
 
     if (menubutton->icon_size > 0)
     {
-        xfce_menubutton_scale (menubutton,
-                               menubutton->icon_size, 
-                               menubutton->icon_size);
+	xfce_menubutton_scale (menubutton);
     }
     else
     {
-        gtk_image_set_from_pixbuf (GTK_IMAGE (menubutton->image), 
-                                   menubutton->pb);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (menubutton->image),
+				   menubutton->pb);
     }
 }
 
@@ -217,97 +229,145 @@ xfce_menubutton_finalize (GObject * object)
     G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void 
+static void
 xfce_menubutton_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
 {
     XfceMenubutton *menubutton;
-    gint xthick, ythick;
-    
+    gint ythick, borderwidth;
+    GtkBorder *tmp_border;
+
     g_return_if_fail (widget != NULL);
     g_return_if_fail (XFCE_IS_MENUBUTTON (widget));
     g_return_if_fail (allocation != NULL);
 
-    menubutton = XFCE_MENUBUTTON(widget);
-    
-    xthick = widget->style->xthickness + menubutton->hbox->style->xthickness;
-    ythick = widget->style->ythickness + menubutton->hbox->style->ythickness;
-    
-    menubutton->icon_size = MIN(allocation->width - 2 * xthick, 
-                                allocation->height - 2 * ythick);
+    menubutton = XFCE_MENUBUTTON (widget);
+
+    borderwidth = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+    ythick = 2 * (CHILD_SPACING + borderwidth + widget->style->ythickness +
+		  menubutton->hbox->style->ythickness +
+		  menubutton->image->style->ythickness);
+
+    gtk_widget_style_get (widget, "default_border", &tmp_border, NULL);
+
+    if (tmp_border && GTK_WIDGET_CAN_DEFAULT (widget))
+    {
+	ythick += tmp_border->top + tmp_border->bottom;
+    }
+
+    if (menubutton->icon_size == allocation->height - ythick)
+    {
+	parent_class->size_allocate (widget, allocation);
+	return;
+    }
+
+    menubutton->icon_size = allocation->height - ythick;
 
     widget->allocation = *allocation;
 
-#if 0
-    g_message("size: %d x %d, scaled: %s", 
-	      allocation->width,
-	      allocation->height,
-	      menubutton->scaled ? "yes" : "no");
+#if DEBUGGING
+    g_print (" ** size: %dn",
+	     menubutton->icon_size);
 #endif
 
-    /* prevent race condition */
-    if (!menubutton->scaled)
+    if (menubutton->pb)
     {
-	if (menubutton->pb)
-	    xfce_menubutton_scale (menubutton,
-                                   menubutton->icon_size, 
-                                   menubutton->icon_size);
-	menubutton->scaled = TRUE;
+        xfce_menubutton_scale (menubutton);
     }
-    else
-    {
-	menubutton->scaled = FALSE;
-    }
-    
+
     parent_class->size_allocate (widget, allocation);
-    
-#if 0
-    g_message("image size: %d x %d", 
-	      menubutton->image->allocation.width,
-	      menubutton->image->allocation.height);
+    gtk_widget_queue_draw(widget);
+
+#if DEBUGGING
+    g_print (" ** allocated size: %d x %d\n",
+	     menubutton->image->allocation.width,
+	     menubutton->image->allocation.height);
 #endif
 }
 
-static void
-xfce_menubutton_scale (XfceMenubutton * menubutton, int width, int height)
+static gboolean
+xfce_menubutton_scale (XfceMenubutton * menubutton)
 {
     GdkPixbuf *tmp;
-    float wratio, hratio;
+    double wratio, hratio;
     int w, h, pw, ph;
 
-    g_return_if_fail (menubutton != NULL);
-    g_return_if_fail (XFCE_IS_MENUBUTTON (menubutton));
-    g_return_if_fail (menubutton->pb != NULL);
-    g_return_if_fail (GDK_IS_PIXBUF (menubutton->pb));
+    g_return_val_if_fail (menubutton != NULL, FALSE);
+    g_return_val_if_fail (XFCE_IS_MENUBUTTON (menubutton), FALSE);
+    g_return_val_if_fail (menubutton->pb != NULL, FALSE);
+    g_return_val_if_fail (GDK_IS_PIXBUF (menubutton->pb), FALSE);
 
     pw = gdk_pixbuf_get_width (menubutton->pb);
     ph = gdk_pixbuf_get_height (menubutton->pb);
 
-    w = width <= 0 ? pw + 1 : width;
-    h = height <= 0 ? ph + 1 : height;
+    w = menubutton->icon_size <= 0 ? pw : menubutton->icon_size;
+    h = menubutton->icon_size <= 0 ? ph : menubutton->icon_size;
 
-    wratio = (float) pw / (float) w;
-    hratio = (float) ph / (float) h;
-/*
-    g_printerr("w = %d, h = %d, pw = %d, ph = %d\n", w, h, pw, ph);
-    g_printerr("wratio = %f, hratio = %f\n", wratio, hratio);
-*/
-    if (wratio <= 1.0 && hratio <= 1.0)
+    if (w >= pw && h >= ph)
     {
-	/* don't scale up -> ugly icons */
-	gtk_image_set_from_pixbuf (GTK_IMAGE (menubutton->image),
-				   menubutton->pb);
-	return;
+#if DEBUGGING
+	g_print (" ** don't scale: %d x %d\n", pw, ph);
+#endif
+	/* don't scale up -> ugly icons */ 
+	tmp = menubutton->pb;
+	g_object_ref (tmp);
     }
-    else if (wratio > hratio)
+    else
     {
-	h = (float) ph / wratio;
-    }
-    else if (hratio > wratio)
-    {
-	w = (float) pw / hratio;
+	wratio = (double) pw / w;
+	hratio = (double) ph / h;
+
+	if (wratio > hratio)
+	    h = rint ((double) ph / wratio);
+	else if (hratio > wratio)
+	    w = rint ((double) pw / hratio);
+
+#if DEBUGGING
+	g_print (" ** scale image: %d x %d\n", w, h);
+#endif
+
+	tmp = gdk_pixbuf_scale_simple (menubutton->pb, w, h,
+				       GDK_INTERP_BILINEAR);
     }
 
-    tmp = gdk_pixbuf_scale_simple (menubutton->pb, w, h, GDK_INTERP_BILINEAR);
     gtk_image_set_from_pixbuf (GTK_IMAGE (menubutton->image), tmp);
     g_object_unref (tmp);
+
+    return TRUE;
+}
+
+static void
+real_style_set (GtkWidget * widget, GtkStyle * old)
+{
+    GtkRcStyle *style;
+
+    style = gtk_widget_get_modifier_style (widget);
+    style->xthickness = style->ythickness = 0;
+    gtk_widget_modify_style (widget, style);
+}
+
+static void
+image_style_set (GtkWidget * widget, GtkStyle * old)
+{
+    static int recursive = 0;
+
+    if (recursive > 0)
+	return;
+
+    recursive++;
+    real_style_set (widget, old);
+    recursive--;
+}
+
+static void
+hbox_style_set (GtkWidget * widget, GtkStyle * old)
+{
+    static int recursive = 0;
+
+    if (recursive > 0)
+	return;
+
+    recursive++;
+    real_style_set (widget, old);
+    recursive--;
 }
